import React, {
  Component,
  PropTypes,
} from 'react';
import {
  View,
  ScrollView,
  Dimensions,
} from 'react-native';
import moment from 'moment';
import Week from './Week';
import Day from './Day';
import styles from './style';

const SCREEN_WIDTH = Dimensions.get('window').width;
const today = moment().format('YYYY-MM-DD');

class CalenderContent extends Component {
    static propTypes = {
      weekStart: PropTypes.number,
      startDate: PropTypes.string,
      endDate: PropTypes.string,
      selectedDate: PropTypes.arrayOf(PropTypes.string),
      onDateClick: PropTypes.func,
      removeClippedSubviews: PropTypes.bool, // ScrollView
      renderDay: PropTypes.func,
      style: PropTypes.shape({
        container: View.propTypes.style,
        week: View.propTypes.style,
      }),
    };

    static defaultProps = {
      startDate: today,
      endDate: null,
      width: SCREEN_WIDTH,
      removeClippedSubviews: true,
      onDateClick: null,
      weekStart: 1,
      renderDay: null,
      selectedDate: null,
      style: null,
    };

    state = {
      currentMoment: moment(this.props.startDate), // 当前要改变的state状态
      selectedMoment: moment(this.props.selectedDate[0]), // 选中的state状态
    };

    /* 生成dataSource */
    getMonthes(currentMoment) {
      let i = 1;
      let nextMonth = moment(currentMoment).startOf('month');
      const res = [nextMonth];
      const endMonth = moment(this.props.endDate).startOf('month');
      while (moment(nextMonth).isBefore(endMonth)) {
        nextMonth = moment(currentMoment).add(i, 'month');
        res.push(nextMonth);
        i += 1;
      }
      return res;
    }

    selectDate(date) {
      this.setState({ selectedMoment: date });
    }

    renderCalendarView(currentMoment) {
      let renderIndex = 0; // 起始渲染位置
      let weekDate = []; // 保存一周的数据
      const selectedMoment = this.state.selectedMoment;
      const weekStart = this.props.weekStart; // 从星期几开始
      const weekRows = []; // 保存每周的数据
      const currStartWeek = currentMoment.isoWeekday() - weekStart; // 每月第一天是从星期几开始做偏移
      const befourOffset = (currStartWeek + 7) % 7; // 偏移之前空几天

      for (renderIndex; renderIndex < 42; renderIndex += 1) {
        const dayIndex = renderIndex - befourOffset;
        const thisMoment = moment(currentMoment).add(dayIndex, 'day'); // 补齐offset的空缺
        const title = thisMoment.format('D') === moment(thisMoment).startOf('month').format('D') ? `${thisMoment.format('M')}月` : thisMoment.format('D');
        const isMonthStart = thisMoment.format('D') === moment(thisMoment).startOf('month').format('D');
        const disabled = moment(thisMoment).isBefore(this.props.endDate);
        const parameter = {
          key: renderIndex,
          title,
          isMonthStart, // 是否是每月的第一天
          style: {},
          date: moment(thisMoment).format('YYYY-MM-DD'), // 回调函数的date
          onPress: () => {
            this.selectDate(thisMoment);
            if (this.props.onDateClick) {
              this.props.onDateClick(thisMoment.format());
            }
          },
          isSelected: selectedMoment.isSame(thisMoment),
          isToday: moment().format('YYYY-MM-DD') === thisMoment.format('YYYY-MM-DD'),
          disabled,
        };
        const element = this.renderDay(parameter);
        weekDate.push(element);
        if (renderIndex % 7 === 6) {
          weekRows.push(<Week style={this.props.style} key={`${renderIndex}-${currentMoment}`}>{weekDate}</Week>);
          weekDate = [];
        }
      }
      return <View style={{ width: SCREEN_WIDTH }} key={`${currentMoment}`}>{weekRows}</View>;
    }


    renderDay(parameter) {
      if (this.props.renderDay) {
        return this.props.renderDay(parameter);
      }
      return <Day {...parameter} />;
    }

    render() {
      const calendarDates = this.getMonthes(this.state.currentMoment);
      const { style } = this.props;
      return (
        <View style={[styles.container, style.container]}>
          <ScrollView
            ref={(calendar) => { this.calendar = calendar; }}
            horizontal
            scrollEnabled
            pagingEnabled
            removeClippedSubviews
            showsHorizontalScrollIndicator={false}
            automaticallyAdjustContentInsets={false}
          >
            {calendarDates.map(month => this.renderCalendarView(month))}
          </ScrollView>
        </View>
      );
    }
}
export default CalenderContent;

