import React, {
  Component,
  PropTypes,
} from 'react';
import {
  View,
  ScrollView,
  Dimensions,
} from 'react-native';
import moment from 'moment';
import Week from './Week';
import Day from './Day';

const wd = Dimensions.get('window').width;
const VIEW_INDEX = 2;

class CalenderContent extends Component {
      static propTypes = {
        weekStart: PropTypes.number,
        startDate: PropTypes.string,
        endDate: PropTypes.string,
        onDateSelect: PropTypes.func,
        onSwipeNext: PropTypes.func,
        onSwipePrev: PropTypes.func,
        removeClippedSubviews: PropTypes.bool, // ScrollView
        scrollEnabled: PropTypes.bool, // ScrollView
        renderDay: PropTypes.func,
      };

      static defaultProps = {
        startDate: moment().format('YYYY-MM-DD'),
        endDate: null,
        customStyle: {},
        width: wd,
        removeClippedSubviews: true,
        scrollEnabled: true,
        onDateSelect: null,
        onSwipeNext: null,
        onSwipePrev: null,
        weekStart: 1,
        renderDay: null,
      };

      state = {
        currentMoment: moment(this.props.startDate), // 当前要改变的state状态
        selectedMoment: moment(this.props.selectedDate), // 选中的state状态
        rowHeight: null,
        forbade: true,
      };
      componentDidMount() {
        // fixes initial scrolling bug on Android
        setTimeout(() => this.scrollToItem(VIEW_INDEX), 0);
      }
      componentWillReceiveProps(nextProps) {
        console.log('nextProps', nextProps);
      }
      componentDidUpdate() {
        if (this.state.forbade) {
          this.scrollToItem(VIEW_INDEX);
        }
      }

      /* 生成dataSource */
      getCalendarData(currentMoment) {
        console.log('重新渲染');
        if (this.props.scrollEnabled) {
          let i = -VIEW_INDEX;
          const res = [];
          for (i; i <= VIEW_INDEX; i += 1) { res.push(moment(currentMoment).add(i, 'month')); }
          return res;
        }
        return [moment(currentMoment)];
      }

      scrollEnded(event) {
        const { startDate, endDate } = this.props;
        const postion = event.nativeEvent.contentOffset.x;
        const currentPage = postion / wd;
        const newMoment = moment(this.state.currentMoment).add(currentPage - VIEW_INDEX, 'month');
        this.setState({ currentMoment: newMoment, forbade: true });
        // const before = moment(newMoment).isAfter(startDate);
        // const after = moment(newMoment).isBefore(moment(endDate).add(-2, 'month'));
        // console.log(after);
        // if (before && after) {
        //   this.setState({ currentMoment: newMoment, forbade: true });
        // } else {
        //   console.log(currentPage);
        //   this.setState({ forbade: false });
        // }
      }

      scrollToItem(itemIndex) {
        const scrollToX = itemIndex * wd;
        if (this.props.scrollEnabled && this.calendar) {
          this.calendar.scrollTo({ y: 0, x: scrollToX, animated: false });
        }
      }

      selectDate(date) {
        console.log(this.props.onDateSelect);
        if (this.props.selectedDate === undefined) {
          this.setState({ selectedMoment: date });
        }
      }

      renderCalendarView(currentMoment) {
        let renderIndex = 0; // 起始渲染位置
        let weekDate = []; // 保存一周的数据
        const selectedMoment = moment(this.state.selectedMoment);
        const weekStart = this.props.weekStart;
        const weekRows = []; // 保存每周的数据
        const currStartWeek = currentMoment.isoWeekday() - weekStart; // 每月第一天是从星期几开始做偏移
        const befourOffset = (currStartWeek + 7) % 7; // 偏移之前空几天

        for (renderIndex; renderIndex < 42; renderIndex += 1) {
          const dayIndex = renderIndex - befourOffset;
          const thisMoment = moment(currentMoment).add(dayIndex, 'day'); // 补齐offset的空缺
          const title = thisMoment.format('D') === moment(thisMoment).startOf('month').format('D') ? `${thisMoment.format('M')}月` : thisMoment.format('D');
          const isMonthStart = thisMoment.format('D') === moment(thisMoment).startOf('month').format('D');
          const props = {
            key: renderIndex,
            title,
            isMonthStart,
            style: {},
            date: moment(thisMoment).format('YYYY-MM-DD'),
            onPress: () => {
              this.selectDate(thisMoment);
              this.props.onDateSelect && this.props.onDateSelect(thisMoment ? thisMoment.format() : null);
            },
            isSelected: selectedMoment.isSame(thisMoment),
            today: moment().format('YYYY-MM-DD') === thisMoment.format('YYYY-MM-DD'),
          };
          const element = this.renderDay(props);
          weekDate.push(element);
          if (renderIndex % 7 === 6) {
            weekRows.push(<Week key={`${renderIndex}-${currentMoment}`}>{weekDate}</Week>);
            weekDate = [];
          }
        }
        return <View style={{ width: wd }} key={`${currentMoment}`}>{weekRows}</View>;
      }

      renderDay(props) {
        if (this.props.renderDay) {
          return this.props.renderDay(props);
        }
        return <Day {...props} />;
      }

      render() {
        const calendarDates = this.getCalendarData(this.state.currentMoment);
        console.log(calendarDates);
        return (
          <View style={{ flex: 1 }}>
            <ScrollView
              ref={(calendar) => { this.calendar = calendar; }}
              horizontal
              scrollEnabled
              pagingEnabled
              removeClippedSubviews
              scrollEventThrottle={1000} // 每1000S被调用1次
              showsHorizontalScrollIndicator={false}
              automaticallyAdjustContentInsets={false}
              onMomentumScrollEnd={event => this.scrollEnded(event)}
            >
              {calendarDates.map(month => this.renderCalendarView(month))}
            </ScrollView>
          </View>
        );
      }
}
export default CalenderContent;
