import React, { PropTypes } from 'react';
import {
  View,
  Text,
} from 'react-native';
import Chart from './Chart';
import styles from './style';

const handleData = (arrX, arrValue, data) => {
  data.forEach((el) => {
    arrX.push(el.x);
    arrValue.push(el.value);
    return null;
  });
  return Math.max(...arrValue);
};

const BarChart = (props) => {
  const {
    data,
    style,
    splitNumber,
    Interval,
  } = props;
  const AxisStyle = {
    AxisX: styles.AxisX,
    AxisY: styles.AxisY,
    labelX: styles.labelX,
    labelY: styles.labelY,
    lineX: styles.lineX,
    lineY: styles.lineY,
  };

  const RectangularStyle = {
    grid: styles.grid,
    bar: styles.bar,
  };
  const arrValue = [];
  const arrX = [];
  const max = handleData(arrX, arrValue, data);

  const onClick = (item) => {
    console.log(item);
  };

  return (
    <View style={{ flex: 1 }}>
      <View style={{ flex: 1, flexDirection: 'row' }}>
        <Chart.AxisY
          max={max}
          style={Object.assign(AxisStyle, style.AxisStyle)}
          splitNumber={splitNumber}
        />
        <Chart>
          {
            data.map(
              (el, idx) => {
                const key = `key_${idx}`;
                return (<Chart.Rectangular
                  onClick={onClick}
                  style={Object.assign(RectangularStyle, style.RectangularStyle)}
                  key={key}
                  data={el}
                  label={{ title: `${el.value}`, style: [styles.barLabel, style.RectangularStyle.barLabel] }}
                  max={max}
                />);
              })
          }
        </Chart>
      </View>
      <Chart.AxisX
        data={arrX}
        style={Object.assign(AxisStyle, style.AxisStyle)}
        Interval={Interval}
      />
    </View>
  );
};

BarChart.propTypes = {
  data: PropTypes.arrayOf(PropTypes.shape({
    id: PropTypes.number,
    x: PropTypes.number,
    value: PropTypes.number,
  })),
  style: PropTypes.shape({
    AxisStyle: PropTypes.shape({
      AxisX: View.propTypes.style,
      lineX: View.propTypes.style,
      labelX: Text.propTypes.style,
      AxisY: View.propTypes.style,
      lineY: View.propTypes.style,
      labelY: Text.propTypes.style,
    }),
    RectangularStyle: PropTypes.shape({
      grid: View.propTypes.style,
      bar: View.propTypes.style,
    }),
  }),
  splitNumber: PropTypes.number,
  Interval: PropTypes.number,
};
BarChart.defaultProps = {
  data: [],
  style: {},
  Interval: 0,
  splitNumber: 1,
};

export default BarChart;

