import React, {
  PropTypes,
  Component,
} from 'react';
import {
  View,
  Text,
  LayoutAnimation,
  TouchableOpacity,
} from 'react-native';
import Label from '../../Label';
import styles from './style';
import config from '../config';

class Rectangular extends Component {
  static propTypes = {
    label: PropTypes.shape({
      title: PropTypes.string,
      style: Text.propTypes.style,
    }),
    data: PropTypes.shape({
      value: PropTypes.number,
      max: PropTypes.number,
    }),
    style: PropTypes.shape({
      grid: View.propTypes.style,
      bar: View.propTypes.style,
    }),
    onClick: PropTypes.func,
    max: PropTypes.number,
  }

  static defaultProps = {
    label: {},
    data: [],
    style: null,
    onClick: null,
    max: 0,
  }

  constructor(props) {
    super(props);
    this.state = {
      height: 0,
      showNumber: false,
    };
  }

  onPress = () => {
    this.props.onClick(this.props.data);
  }

  layout = (e) => {
    LayoutAnimation.linear();
    let height = (this.props.data.value / this.props.max) * e.layout.height;
    height = height > e.layout.height - config.LABEL_FS ? height - config.LABEL_FS : height;
    this.setState({
      height,
      showNumber: true,
    });
  }

  render() {
    const {
      label,
      style,
    } = this.props;
    return (
      <TouchableOpacity onPress={this.onPress}>
        <View
          style={[styles.grid, style.grid]}
          onLayout={({ nativeEvent: e }) => this.layout(e)}
        >
          { label && this.state.showNumber && <Label label={label} /> }
          <View style={[styles.bar, style.bar, { height: this.state.height }]} />
        </View>
      </TouchableOpacity>
    );
  }
}

export default Rectangular;

